﻿Namespace EIBData.DPT
    Public MustInherit Class EIBDataDPT_13_Base
        Inherits EIBDataDPT_Base

        Protected _value As Int32 = Nothing


        Sub New(data As Byte())
            If Not CanEncode(data) Then Throw New EIB.Exceptions.EIBDataDecodeException("")

            MyBase._data = data
            _value = (data(1) << 24) + (data(2) << 16) + (data(3) << 8) + data(4)
        End Sub

        Sub New(value As Int32)
            _value = value
            MyBase._data = New Byte() {0, ((_value And &HFF000000) >> 24), ((_value And &HFF0000) >> 16), ((_value And &HFF00) >> 8), (_value And &HFF)}
        End Sub


        Public Overrides Function CanEncode(data() As Byte) As Boolean
            Return data IsNot Nothing AndAlso data.Length = DPT_OctetLength
        End Function

        Public Overrides Function ToString() As String
            Return _value
        End Function

        Public Overrides Function ToObject() As Object()
            Return New Object() {_value}
        End Function

        Public Overrides ReadOnly Property ObjectType As System.Type()
            Get
                Return New Type() {GetType(Int32)}
            End Get
        End Property

        Public Overrides Function ToInfoList() As List(Of EIB.InfoBase)
            Dim list As New List(Of EIB.InfoBase)

            list.Add(New EIB.Info("Type", DPT_Name))
            list.Add(New EIB.Info("Value", ToString))

            Return list
        End Function

        Public Shared Shadows ReadOnly Property DPT_BaseName As String
            Get
                Return "V32"
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_OctetLength As Byte
            Get
                Return 5
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_Name As String
            Get
                Return "DPT_13"
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_Number As String
            Get
                Return "13.xxx"
            End Get
        End Property

        Public ReadOnly Property Value As Int32
            Get
                Return _value
            End Get
        End Property
    End Class
End Namespace