﻿Namespace EIBData.DPT
    Public MustInherit Class EIBDataDPT_18_Base
        Inherits EIBDataDPT_Base

        Protected _control As Boolean = Nothing
        Protected _scene As Byte = Nothing


        Sub New(data As Byte())
            If Not CanEncode(data) Then Throw New EIB.Exceptions.EIBDataDecodeException("")

            MyBase._data = data
            _control = ((data(1) And &H80) >> 7)
            _scene = (data(1) And &H3F)
        End Sub

        Sub New(control As Boolean, scene As Byte)
            If scene > &H3F Then Throw New EIB.Exceptions.EIBDataEncodeException("")

            _control = control
            _scene = scene
            MyBase._data = New Byte() {0, (If(control, 1, 0) << 7) + (scene And &H3F)}
        End Sub


        Public Overrides Function CanEncode(data() As Byte) As Boolean
            Return data IsNot Nothing AndAlso data.Length = DPT_OctetLength
        End Function

        Public Overrides Function ToString() As String
            Return String.Format("Control: {0}, Scene: {1}", _control, _scene)
        End Function

        Public Overrides Function ToObject() As Object()
            Return New Object() {_control, _scene}
        End Function

        Public Overrides ReadOnly Property ObjectType As System.Type()
            Get
                Return New Type() {GetType(Boolean), GetType(Byte)}
            End Get
        End Property

        Public Overrides Function ToInfoList() As List(Of EIB.InfoBase)
            Dim list As New List(Of EIB.InfoBase)

            list.Add(New EIB.Info("Type", DPT_Name))
            list.Add(New EIB.Info("Control", _control))
            list.Add(New EIB.Info("SceneNumber", _scene))

            Return list
        End Function

        Public Shared Shadows ReadOnly Property DPT_BaseName As String
            Get
                Return "B1r1U6"
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_OctetLength As Byte
            Get
                Return 2
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_Name As String
            Get
                Return "DPT_18"
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_Number As String
            Get
                Return "18.xxx"
            End Get
        End Property

        Public ReadOnly Property Control As Boolean
            Get
                Return _control
            End Get
        End Property

        Public ReadOnly Property SceneNumber As Byte
            Get
                Return _scene
            End Get
        End Property
    End Class
End Namespace