﻿Namespace EIBData.DPT
    Public MustInherit Class EIBDataDPT_9_Base
        Inherits EIBDataDPT_Base

        Protected _value As Double = Nothing


        Sub New(data As Byte())
            If Not CanEncode(data) Then Throw New EIB.Exceptions.EIBDataDecodeException("")

            MyBase._data = data

            Dim s As Boolean = (data(1) And &H80) >> 7
            Dim e As Byte = (data(1) And &H78) >> 3
            Dim m As UInt16 = (data(1) And &H7) << 8 + data(2)

            _value = -1 ^ s * 0.01 * m * 2 ^ e
        End Sub

        Sub New(value As Double)
            If value < -671088.64 Then Throw New EIB.Exceptions.EIBDataEncodeException("")
            If value > 670760.96 Then Throw New EIB.Exceptions.EIBDataEncodeException("")

            _value = value

            Dim s As Boolean = If(_value < 0, True, False)
            Dim e As Byte = 0
            Dim m As UInt16 = 0

            MyBase._data = New Byte() {0, (If(s, 1, 0) << 7) + (e << 3) + ((m And &H700) >> 8), (m And &HFF)}
        End Sub


        Public Overrides Function CanEncode(data() As Byte) As Boolean
            Return data IsNot Nothing AndAlso data.Length = DPT_OctetLength
        End Function

        Public Overrides Function ToString() As String
            Return _value
        End Function

        Public Overrides Function ToObject() As Object()
            Return New Object() {_value}
        End Function

        Public Overrides ReadOnly Property ObjectType As System.Type()
            Get
                Return New Type() {GetType(Double)}
            End Get
        End Property

        Public Overrides Function ToInfoList() As List(Of EIB.InfoBase)
            Dim list As New List(Of EIB.InfoBase)

            list.Add(New EIB.Info("Type", DPT_Name))
            list.Add(New EIB.Info("Value", _value))

            Return list
        End Function

        Public Shared Shadows ReadOnly Property DPT_BaseName As String
            Get
                Return "F16"
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_OctetLength As Byte
            Get
                Return 3
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_Name As String
            Get
                Return "DPT_9"
            End Get
        End Property

        Public Shared Shadows ReadOnly Property DPT_Number As String
            Get
                Return "9.xxx"
            End Get
        End Property

        Public ReadOnly Property Value As Double
            Get
                Return _value
            End Get
        End Property
    End Class
End Namespace