﻿Namespace EIBData.EIS
    Public NotInheritable Class EIBDataEIS_11
        Inherits EIBDataEIS_Base

        Protected _value As Int32 = Nothing
        Protected _Uvalue As UInt32 = Nothing


        Sub New(data As Byte())
            If Not CanEncode(data) Then Throw New EIB.Exceptions.EIBDataDecodeException("")

            MyBase._data = data
            _value = (data(1) << 24) + (data(2) << 16) + (data(3) << 8) + data(4)
        End Sub

        Sub New(value As Int32)
            _value = value
            _Uvalue = Nothing
            MyBase._data = New Byte() {0, ((_value And &HFF000000) >> 24), ((_value And &HFF0000) >> 16), ((_value And &HFF00) >> 8), (_value And &HFF)}
        End Sub

        Sub New(value As UInt32)
            _value = Nothing
            _Uvalue = value
            MyBase._data = New Byte() {0, ((_Uvalue And &HFF000000) >> 24), ((_Uvalue And &HFF0000) >> 16), ((_Uvalue And &HFF00) >> 8), (_Uvalue And &HFF)}
        End Sub


        Public Overrides Function CanEncode(data() As Byte) As Boolean
            Return data IsNot Nothing AndAlso data.Length = EIS_OctetLength
        End Function

        Public Overrides Function ToString() As String
            Return _value
        End Function

        Public Overrides Function ToObject() As Object()
            Return New Object() {_value}
        End Function

        Public Overrides ReadOnly Property ObjectType As System.Type()
            Get
                Return New Type() {GetType(Int32)}
            End Get
        End Property

        Public Overrides Function ToInfoList() As List(Of EIB.InfoBase)
            Dim list As New List(Of EIB.InfoBase)

            list.Add(New EIB.Info("Type", EIS_Name))
            list.Add(New EIB.Info("Value (signed)", _value))
            list.Add(New EIB.Info("Value (unsigned)", _Uvalue))

            Return list
        End Function

        Public Shared Shadows ReadOnly Property EIS_OctetLength As Byte
            Get
                Return 5
            End Get
        End Property

        Public Shared Shadows ReadOnly Property EIS_Name As String
            Get
                Return "EIS 11"
            End Get
        End Property


        Public ReadOnly Property Value As Int32
            Get
                Return _value
            End Get
        End Property

        Public ReadOnly Property UValue As UInt32
            Get
                Return _Uvalue
            End Get
        End Property
    End Class
End Namespace
