﻿Namespace EIBData.EIS
    Public NotInheritable Class EIBDataEIS_8
        Inherits EIBDataEIS_Base

        Protected _control As Boolean = Nothing
        Protected _value As Boolean = Nothing


        Sub New(data As Byte())
            If Not CanEncode(data) Then Throw New EIB.Exceptions.EIBDataDecodeException("")

            MyBase._data = data
            _control = (data(0) And &H2) >> 1
            _value = (data(0) And &H1)
        End Sub

        Sub New(control As Boolean, value As Boolean)
            _control = control
            _value = value
            MyBase._data = New Byte() {(If(_control, 1, 0) << 1) + If(_value, 1, 0)}
        End Sub


        Public Overrides Function CanEncode(data() As Byte) As Boolean
            Return data IsNot Nothing AndAlso data.Length = EIS_OctetLength
        End Function

        Public Overrides Function ToString() As String
            If _control Then
                Return String.Format("Control. Function value {0}", If(_value, 1, 0))
            Else
                Return "No control"
            End If
        End Function

        Public Overrides Function ToObject() As Object()
            Return New Object() {_control, _value}
        End Function

        Public Overrides ReadOnly Property ObjectType As System.Type()
            Get
                Return New Type() {GetType(Boolean), GetType(Boolean)}
            End Get
        End Property

        Public Overrides Function ToInfoList() As List(Of EIB.InfoBase)
            Dim list As New List(Of EIB.InfoBase)

            list.Add(New EIB.Info("Type", EIS_Name))
            list.Add(New EIB.Info("Control", _control))
            list.Add(New EIB.Info("Value", _value))

            Return list
        End Function

        Public Shared Shadows ReadOnly Property EIS_OctetLength As Byte
            Get
                Return 1
            End Get
        End Property

        Public Shared Shadows ReadOnly Property EIS_Name As String
            Get
                Return "EIS 8"
            End Get
        End Property


        Public ReadOnly Property Control As Boolean
            Get
                Return _control
            End Get
        End Property

        Public ReadOnly Property Value As Boolean
            Get
                Return _value
            End Get
        End Property
    End Class
End Namespace